"""file: merge.py -- correction td5, exercise about merge sort.

"""

import random
import matplotlib.pyplot as plt
import time

# merging function: the merging of two sorted list is the sorted union
# of the two lists.
#
# format: l0 and l1 are sorted lists; merge(l0,l1) returns the merging
# of l0 and l1.
def merge(l0,l1):
  if len(l0) == 0 or len(l1) == 0: return l0+l1
  elif l0[0] < l1[0]: return [l0[0]]+merge(l0[1:],l1)
  else: return [l1[0]]+merge(l0,l1[1:])

# same format as merge. It is just the merge function but in an
# iterative fashion. (It permits to run the merge on big list, as
# Python has very limited features concerning recursion...)
def merge_it(l0,l1):
  li = []
  while(l0 != [] and l1 != []):
    if l0[0]<l1[0]:
      li = li + [l0[0]]
      l0 = l0[1:]
    else:
      li = li + [l1[0]]
      l1 = l1[1:]
  return li+l0+l1

# format: l is a list; mergesort(l) returns a sorted list with the
# same elements as l. Obviously, it uses the merge sort algorithm:
# break the list in two and mergesort each of them, then merge the two
# result back together.
def mergesort(l):
  n = len(l)
  return l if n==0 or n==1 else merge(mergesort(l[:n//2]),mergesort(l[n//2:]))

# same format as mergesort. It is the iterative version of the
# mergesort. See how it is much more elaborated than the simpler
# recursive version.
def mergesort_it(l):
  n = len(l)
  l = map(lambda x: [x],l)
  while n > 1:
    if n % 2 == 1:
      l = l[:-2] + [merge_it(l[-2],l[-1])]
    for i in range(n//2):
      l = l[:i] + [merge_it(l[i],l[i+1])] + l[i+2:]
    n = n//2
  return l[0]

## recall bubble sort from td2

"""bubble_up(li,swap) makes the maximum of li[:swap] go to the cell
indexed by swap-1. It returns the new last swap position after it.

"""
def bubble_up(li,swap):
  new = swap-1 # this is the maximum it can be
  for j in range(swap-1):
    if li[j] > li[j+1]:
      li[j],li[j+1] = li[j+1],li[j]
      new = j+1
  return new

"""bubble_sort(li) returns a sorted version of 'li' by the method of
bubbling.

"""
def bubble_sort(li):
  tmp = li[:] # copy li
  lswap = len(li) # remember the last swap
  while lswap > 1:
    lswap = bubble_up(tmp,lswap)
  return tmp

## testing and comparing

mi,ma = 1,1000 # interval of tests
num = 10 # number of test
avr = 100 # number of test to make an average of
xs = range(mi,ma,(ma-mi)/num) # equidistant x's in number num inside
                              # [mi,ma[
ysbubble,ysmerge = [],[]
for x in xs:
  ybub,ymer = 0,0
  for k in xrange(avr):
    li = [random.randrange(x) for k in range(x)]
    start = time.clock()
    bubble_sort(li)
    stop = time.clock()
    ybub += stop - start
    start = time.clock()
    mergesort_it(li)
    stop = time.clock()
    ymer += stop - start
  ybub,ymer = ybub/avr,ymer/avr
  ysbubble.append(ybub)
  ysmerge.append(ymer)
plt.plot(xs,ysbubble,color='b')
plt.plot(xs,ysmerge,color='r')
plt.show()
